<?php

namespace Drupal\saudi_identity_field\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use top7up\validateSAID;

/**
 * Plugin implementation of the 'saudi_identity_default' widget.
 *
 * @FieldWidget(
 *   id = "saudi_identity_default",
 *   label = @Translation("Saudi Identity"),
 *   field_types = {
 *     "saudi_identity"
 *   }
 * )
 */
class SaudiIdentityDefaultWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'placeholder' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element['placeholder'] = [
      '#type' => 'textfield',
      '#title' => t('Placeholder'),
      '#default_value' => $this->getSetting('placeholder'),
      '#description' => t('Text that will be shown inside the field until a value is entered. This hint is usually a sample value or a brief description of the expected format.'),
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $placeholder = $this->getSetting('placeholder');
    if (!empty($placeholder)) {
      $summary[] = t('Placeholder: @placeholder', ['@placeholder' => $placeholder]);
    }
    else {
      $summary[] = t('No placeholder');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    // @TODO: We have to load these options from this field settings to show
    // only selected types.
    $saudi_identity_options = [
      'saudi' => t('Saudi Identity'),
      'muqeem' => t('Muqeem  Identity'),
      'gcc' => t('GCC'),
    ];

    $allowed_id_types = [];
    if ($items->getSetting('id_types')) {
      // @TODO: Please review this part.
      foreach($items->getSetting('id_types') as $id_type => $value) {
        if (($id_type == $value) && ($value)) {
          $allowed_id_types[$value] = $saudi_identity_options[$value];
        }
      }
    }

    // Clean version of this element.
    $clean_element = $element;
    $element['saudi_identity'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Saudi ID'),
    ];
    $element['saudi_identity']['type'] = [
      '#type' => 'radios',
      '#required' => $element['#required'],
      '#title' => $this->t('ID type'),
      '#options' => $allowed_id_types,
      '#default_value' => isset($items[$delta]->type) ? $items[$delta]->type : NULL,
      // Here no need to have an element_validate.
    ];
    $element['saudi_identity']['value'] = $clean_element + [
      '#type' => 'textfield',
      '#placeholder' => $this->getSetting('placeholder'),
      '#default_value' => isset($items[$delta]->value) ? $items[$delta]->value : NULL,
      '#element_validate' => [
        [$this, 'validate'],
      ],
    ];
    return $element;
  }

  /**
   * Validate field.
   */
  public function validate(&$element, FormStateInterface &$form_state, $form) {
    // @TODO: Fix this issue to support more than one field.
    // @TODO: Get field name the right way.
    $field_name = $element['#array_parents'][0];
    $field_values = $form_state->getValue([$field_name]);
    $id_type = $field_values[0]['saudi_identity']['type'];
    $id_number = $field_values[0]['saudi_identity']['value'];
    $validation = FALSE;
    switch ($id_type) {
      case 'muqeem':
        // Making sure the muqeem ID number is 10 digits and starting with 2.
        if (2 == substr($id_number, 0, 1) && (strlen($id_number) == 10)) {
          $validation = TRUE;
        }
        break;

      case 'gcc':
        // Currently there are no validation for this field type.
        if (is_numeric($id_number) && (strlen($id_number) > 0))
        $validation = TRUE;
        break;

      case 'saudi':
        // Check Saudi ID number using the needful algorithm.
        if (validateSAID::check($id_number) == 1) {
          $validation = TRUE;
        }
        break;

      // For default we will do nothing.
      default:
        break;
    }

    // @TODO: We have to make sure this value is unique if the settings unique
    // has been selected.
    // Make sure the value validation is not FALSE.
    if ($validation) {
      // Validated.
    }
    else {
      // @TODO: Enhance the error message.
      if ($element['#required'] || (strlen($id_number) > 0) || (strlen($id_type) > 0)) {
        $form_state->setError($element, $this->t('Please provide a valid ID number.'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    foreach ($values as $delta => $value) {
      $values[$delta]['value'] = $value['saudi_identity']['value'];
      $values[$delta]['type'] = $value['saudi_identity']['type'];
    }
    return $values;
  }

}
